@@ -25,12 +25,16 @@ actions.
 import cmd
 import errno
 import getpass
-import xmlrpclib
 import socket
-import urlparse
 import sys
 import threading
 
+from supervisor.compat import xmlrpclib
+from supervisor.compat import urlparse
+from supervisor.compat import unicode
+from supervisor.compat import raw_input
+from supervisor.compat import as_string
+
 from supervisor.medusa import asyncore_25 as asyncore
 
 from supervisor.options import ClientOptions
@@ -40,6 +44,23 @@ from supervisor import xmlrpc
 from supervisor import states
 from supervisor import http_client
 
+class LSBInitExitStatuses:
+    SUCCESS = 0
+    GENERIC = 1
+    INVALID_ARGS = 2
+    UNIMPLEMENTED_FEATURE = 3
+    INSUFFICIENT_PRIVILEGES = 4
+    NOT_INSTALLED = 5
+    NOT_RUNNING = 7
+
+class LSBStatusExitStatuses:
+    NOT_RUNNING = 3
+    UNKNOWN = 4
+
+DEAD_PROGRAM_FAULTS = (xmlrpc.Faults.SPAWN_ERROR,
+                       xmlrpc.Faults.ABNORMAL_TERMINATION,
+                       xmlrpc.Faults.NOT_RUNNING)
+
 class fgthread(threading.Thread):
     """ A subclass of threading.Thread, with a kill() method.
     To be used for foreground output/error streaming.
@@ -104,6 +125,7 @@ class Controller(cmd.Cmd):
         self.options.plugins = []
         self.vocab = ['help']
         self._complete_info = None
+        self.exitstatus = LSBInitExitStatuses.SUCCESS
         cmd.Cmd.__init__(self, completekey, stdin, stdout)
         for name, factory, kwargs in self.options.plugin_factories:
             plugin = factory(self, **kwargs)
@@ -117,6 +139,10 @@ class Controller(cmd.Cmd):
         # We don't want a blank line to repeat the last command.
         return
 
+    def default(self, line):
+        self.output('*** Unknown syntax: %s' % line)
+        self.exitstatus = LSBInitExitStatuses.GENERIC
+
     def exec_cmdloop(self, args, options):
         try:
             import readline
@@ -149,18 +175,19 @@ class Controller(cmd.Cmd):
             self.output('')
             pass
 
+    def set_exitstatus_from_xmlrpc_fault(self, faultcode, ignored_faultcode=None):
+        if faultcode in (ignored_faultcode, xmlrpc.Faults.SUCCESS):
+            pass
+        elif faultcode in DEAD_PROGRAM_FAULTS:
+            self.exitstatus = LSBInitExitStatuses.NOT_RUNNING
+        else:
+            self.exitstatus = LSBInitExitStatuses.GENERIC
+
     def onecmd(self, line):
         """ Override the onecmd method to:
           - catch and print all exceptions
-          - allow for composite commands in interactive mode (foo; bar)
           - call 'do_foo' on plugins rather than ourself
         """
-        origline = line
-        lines = line.split(';') # don't filter(None, line.split), as we pop
-        line = lines.pop(0)
-        # stuffing the remainder into cmdqueue will cause cmdloop to
-        # call us again for each command.
-        self.cmdqueue.extend(lines)
         cmd, arg, line = self.parseline(line)
         if not line:
             return self.emptyline()
@@ -178,7 +205,7 @@ class Controller(cmd.Cmd):
             try:
                 try:
                     return do_func(arg)
-                except xmlrpclib.ProtocolError, e:
+                except xmlrpclib.ProtocolError as e:
                     if e.errcode == 401:
                         if self.options.interactive:
                             self.output('Server requires authentication')
@@ -187,20 +214,19 @@ class Controller(cmd.Cmd):
                             self.output('')
                             self.options.username = username
                             self.options.password = password
-                            return self.onecmd(origline)
+                            return self.onecmd(line)
                         else:
-                            self.options.usage('Server requires authentication')
+                            self.output('Server requires authentication')
+                            self.exitstatus = LSBInitExitStatuses.GENERIC
                     else:
+                        self.exitstatus = LSBInitExitStatuses.GENERIC
                         raise
                 do_func(arg)
-            except SystemExit:
-                raise
             except Exception:
                 (file, fun, line), t, v, tbinfo = asyncore.compact_traceback()
                 error = 'error: %s, %s: file: %s line: %s' % (t, v, file, line)
                 self.output(error)
-                if not self.options.interactive:
-                    sys.exit(2)
+                self.exitstatus = LSBInitExitStatuses.GENERIC
 
     def _get_do_func(self, cmd):
         func_name = 'do_' + cmd
@@ -237,8 +263,9 @@ class Controller(cmd.Cmd):
                     'Sorry, this version of supervisorctl expects to '
                     'talk to a server with API version %s, but the '
                     'remote version is %s.' % (rpcinterface.API_VERSION, api))
+                self.exitstatus = LSBInitExitStatuses.NOT_INSTALLED
                 return False
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
             if e.faultCode == xmlrpc.Faults.UNKNOWN_METHOD:
                 self.output(
                     'Sorry, supervisord responded but did not recognize '
@@ -246,15 +273,20 @@ class Controller(cmd.Cmd):
                     'uses to control it.  Please check that the '
                     '[rpcinterface:supervisor] section is enabled in the '
                     'configuration file (see sample.conf).')
+                self.exitstatus = LSBInitExitStatuses.UNIMPLEMENTED_FEATURE
                 return False
+            self.exitstatus = LSBInitExitStatuses.GENERIC
             raise
-        except socket.error, e:
+        except socket.error as e:
             if e.args[0] == errno.ECONNREFUSED:
                 self.output('%s refused connection' % self.options.serverurl)
+                self.exitstatus = LSBInitExitStatuses.INSUFFICIENT_PRIVILEGES
                 return False
             elif e.args[0] == errno.ENOENT:
                 self.output('%s no such file' % self.options.serverurl)
+                self.exitstatus = LSBInitExitStatuses.NOT_RUNNING
                 return False
+            self.exitstatus = LSBInitExitStatuses.GENERIC
             raise
         return True
 
@@ -268,18 +300,15 @@ class Controller(cmd.Cmd):
             import readline
             line = readline.get_line_buffer()
 
-        # take the last phrase from a line like "stop foo; start bar"
-        phrase = line.split(';')[-1]
-
         matches = []
-        # blank phrase completes to action list
-        if not phrase.strip():
+        # blank line completes to action list
+        if not line.strip():
             matches = self._complete_actions(text)
         else:
-            words = phrase.split()
+            words = line.split()
             action = words[0]
             # incomplete action completes to action list
-            if len(words) == 1 and not phrase.endswith(' '):
+            if len(words) == 1 and not line.endswith(' '):
                 matches = self._complete_actions(text)
             # actions that accept an action name
             elif action in ('help'):
@@ -444,11 +473,13 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if len(args) < 1:
             self.ctl.output('Error: too few arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_tail()
             return
 
         elif len(args) > 3:
             self.ctl.output('Error: too many arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_tail()
             return
 
@@ -466,9 +497,11 @@ class DefaultControllerPlugin(ControllerPluginBase):
                 channel = args[-1].lower()
                 if channel not in ('stderr', 'stdout'):
                     self.ctl.output('Error: bad channel %r' % channel)
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     return
             else:
                 self.ctl.output('Error: tail requires process name')
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 return
 
         bytes = 1600
@@ -482,6 +515,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
                     bytes = int(what)
                 except:
                     self.ctl.output('Error: bad argument %s' % modifier)
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     return
 
         supervisor = self.ctl.get_supervisor()
@@ -497,7 +531,8 @@ class DefaultControllerPlugin(ControllerPluginBase):
                 else: # if channel is 'stderr'
                     output = supervisor.readProcessStderrLog(name,
                                                              -bytes, 0)
-            except xmlrpclib.Fault, e:
+            except xmlrpclib.Fault as e:
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 template = '%s: ERROR (%s)'
                 if e.faultCode == xmlrpc.Faults.NO_FILE:
                     self.ctl.output(template % (name, 'no log file'))
@@ -530,6 +565,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if len(args) > 1:
             self.ctl.output('Error: too many arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_maintail()
             return
 
@@ -543,11 +579,13 @@ class DefaultControllerPlugin(ControllerPluginBase):
                     what = int(what)
                 except:
                     self.ctl.output('Error: bad argument %s' % args[0])
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     return
                 else:
                     bytes = what
             else:
                 self.ctl.output('Error: bad argument %s' % args[0])
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 return
 
         else:
@@ -557,7 +595,8 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         try:
             output = supervisor.readLog(-bytes, 0)
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             template = '%s: ERROR (%s)'
             if e.faultCode == xmlrpc.Faults.NO_FILE:
                 self.ctl.output(template % ('supervisord', 'no log file'))
@@ -578,7 +617,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
             )
 
     def do_quit(self, arg):
-        sys.exit(0)
+        return self.ctl.do_EOF(arg)
 
     def help_quit(self):
         self.ctl.output("quit\tExit the supervisor shell.")
@@ -603,13 +642,17 @@ class DefaultControllerPlugin(ControllerPluginBase):
             self.ctl.output(line)
 
     def do_status(self, arg):
+        # XXX In case upcheck fails, we override the exitstatus which
+        # should only return 4 for do_status
+        # TODO review this
         if not self.ctl.upcheck():
+            self.ctl.exitstatus = LSBStatusExitStatuses.UNKNOWN
             return
 
         supervisor = self.ctl.get_supervisor()
         all_infos = supervisor.getAllProcessInfo()
 
-        names = arg.split()
+        names = as_string(arg).split()
         if not names or "all" in names:
             matching_infos = all_infos
         else:
@@ -634,8 +677,13 @@ class DefaultControllerPlugin(ControllerPluginBase):
                     else:
                         msg = "%s: ERROR (no such process)" % name
                     self.ctl.output(msg)
+                    self.ctl.exitstatus = LSBStatusExitStatuses.UNKNOWN
         self._show_statuses(matching_infos)
 
+        for info in matching_infos:
+            if info['state'] in states.STOPPED_STATES:
+                self.ctl.exitstatus = LSBStatusExitStatuses.NOT_RUNNING
+
     def help_status(self):
         self.ctl.output("status <name>\t\tGet status for a single process")
         self.ctl.output("status <gname>:*\tGet status for all "
@@ -659,13 +707,17 @@ class DefaultControllerPlugin(ControllerPluginBase):
             for name in names:
                 try:
                     info = supervisor.getProcessInfo(name)
-                except xmlrpclib.Fault, e:
+                except xmlrpclib.Fault as e:
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     if e.faultCode == xmlrpc.Faults.BAD_NAME:
                         self.ctl.output('No such process %s' % name)
                     else:
                         raise
                 else:
-                    self.ctl.output(str(info['pid']))
+                    pid = info['pid']
+                    self.ctl.output(str(pid))
+                    if pid == 0:
+                        self.ctl.exitstatus = LSBInitExitStatuses.NOT_RUNNING
 
     def help_pid(self):
         self.ctl.output("pid\t\t\tGet the PID of supervisord.")
@@ -704,15 +756,15 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if not names:
             self.ctl.output("Error: start requires a process name")
+            self.ctl.exitstatus = LSBInitExitStatuses.INVALID_ARGS
             self.help_start()
             return
 
         if 'all' in names:
             results = supervisor.startAllProcesses()
             for result in results:
-                result = self._startresult(result)
-                self.ctl.output(result)
-
+                self.ctl.output(self._startresult(result))
+                self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'], xmlrpc.Faults.ALREADY_STARTED)
         else:
             for name in names:
                 group_name, process_name = split_namespec(name)
@@ -720,23 +772,26 @@ class DefaultControllerPlugin(ControllerPluginBase):
                     try:
                         results = supervisor.startProcessGroup(group_name)
                         for result in results:
-                            result = self._startresult(result)
-                            self.ctl.output(result)
-                    except xmlrpclib.Fault, e:
+                            self.ctl.output(self._startresult(result))
+                            self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'], xmlrpc.Faults.ALREADY_STARTED)
+                    except xmlrpclib.Fault as e:
                         if e.faultCode == xmlrpc.Faults.BAD_NAME:
                             error = "%s: ERROR (no such group)" % group_name
                             self.ctl.output(error)
+                            self.ctl.exitstatus = LSBInitExitStatuses.INVALID_ARGS
                         else:
+                            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                             raise
                 else:
                     try:
                         result = supervisor.startProcess(name)
-                    except xmlrpclib.Fault, e:
-                        error = self._startresult({'status': e.faultCode,
-                                                   'name': process_name,
-                                                   'group': group_name,
-                                                   'description': e.faultString})
-                        self.ctl.output(error)
+                    except xmlrpclib.Fault as e:
+                        error = {'status': e.faultCode,
+                                  'name': process_name,
+                                  'group': group_name,
+                                  'description': e.faultString}
+                        self.ctl.output(self._startresult(error))
+                        self.ctl.set_exitstatus_from_xmlrpc_fault(error['status'], xmlrpc.Faults.ALREADY_STARTED)
                     else:
                         name = make_namespec(group_name, process_name)
                         self.ctl.output('%s: started' % name)
@@ -778,25 +833,27 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if not names:
             self.ctl.output('Error: stop requires a process name')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_stop()
             return
 
         if 'all' in names:
             results = supervisor.stopAllProcesses()
             for result in results:
-                result = self._stopresult(result)
-                self.ctl.output(result)
-
+                self.ctl.output(self._stopresult(result))
+                self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'], xmlrpc.Faults.NOT_RUNNING)
         else:
             for name in names:
                 group_name, process_name = split_namespec(name)
                 if process_name is None:
                     try:
                         results = supervisor.stopProcessGroup(group_name)
+
                         for result in results:
-                            result = self._stopresult(result)
-                            self.ctl.output(result)
-                    except xmlrpclib.Fault, e:
+                            self.ctl.output(self._stopresult(result))
+                            self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'], xmlrpc.Faults.NOT_RUNNING)
+                    except xmlrpclib.Fault as e:
+                        self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                         if e.faultCode == xmlrpc.Faults.BAD_NAME:
                             error = "%s: ERROR (no such group)" % group_name
                             self.ctl.output(error)
@@ -804,13 +861,14 @@ class DefaultControllerPlugin(ControllerPluginBase):
                             raise
                 else:
                     try:
-                        result = supervisor.stopProcess(name)
-                    except xmlrpclib.Fault, e:
-                        error = self._stopresult({'status': e.faultCode,
-                                                  'name': process_name,
-                                                  'group': group_name,
-                                                  'description':e.faultString})
-                        self.ctl.output(error)
+                        supervisor.stopProcess(name)
+                    except xmlrpclib.Fault as e:
+                        error = {'status': e.faultCode,
+                                 'name': process_name,
+                                 'group': group_name,
+                                 'description':e.faultString}
+                        self.ctl.output(self._stopresult(error))
+                        self.ctl.set_exitstatus_from_xmlrpc_fault(error['status'], xmlrpc.Faults.NOT_RUNNING)
                     else:
                         name = make_namespec(group_name, process_name)
                         self.ctl.output('%s: stopped' % name)
@@ -830,6 +888,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
             self.ctl.output(
                 'Error: signal requires a signal name and a process name')
             self.help_signal()
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             return
 
         sig = args[0]
@@ -838,10 +897,10 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if 'all' in names:
             results = supervisor.signalAllProcesses(sig)
-            for result in results:
-                result = self._signalresult(result)
-                self.ctl.output(result)
 
+            for result in results:
+                self.ctl.output(self._signalresult(result))
+                self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'])
         else:
             for name in names:
                 group_name, process_name = split_namespec(name)
@@ -851,23 +910,25 @@ class DefaultControllerPlugin(ControllerPluginBase):
                             group_name, sig
                             )
                         for result in results:
-                            result = self._signalresult(result)
-                            self.ctl.output(result)
-                    except xmlrpclib.Fault, e:
+                            self.ctl.output(self._signalresult(result))
+                            self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'])
+                    except xmlrpclib.Fault as e:
                         if e.faultCode == xmlrpc.Faults.BAD_NAME:
                             error = "%s: ERROR (no such group)" % group_name
                             self.ctl.output(error)
+                            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                         else:
                             raise
                 else:
                     try:
                         supervisor.signalProcess(name, sig)
-                    except xmlrpclib.Fault, e:
-                        error = self._signalresult({'status': e.faultCode,
-                                                    'name': process_name,
-                                                    'group': group_name,
-                                                    'description':e.faultString})
-                        self.ctl.output(error)
+                    except xmlrpclib.Fault as e:
+                        error = {'status': e.faultCode,
+                                 'name': process_name,
+                                 'group': group_name,
+                                 'description':e.faultString}
+                        self.ctl.output(self._signalresult(error))
+                        self.ctl.set_exitstatus_from_xmlrpc_fault(error['status'])
                     else:
                         name = make_namespec(group_name, process_name)
                         self.ctl.output('%s: signalled' % name)
@@ -886,6 +947,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if not names:
             self.ctl.output('Error: restart requires a process name')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_restart()
             return
 
@@ -913,12 +975,14 @@ class DefaultControllerPlugin(ControllerPluginBase):
             supervisor = self.ctl.get_supervisor()
             try:
                 supervisor.shutdown()
-            except xmlrpclib.Fault, e:
+            except xmlrpclib.Fault as e:
                 if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                     self.ctl.output('ERROR: already shutting down')
                 else:
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     raise
-            except socket.error, e:
+            except socket.error as e:
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 if e.args[0] == errno.ECONNREFUSED:
                     msg = 'ERROR: %s refused connection (already shut down?)'
                     self.ctl.output(msg % self.ctl.options.serverurl)
@@ -936,6 +1000,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
     def do_reload(self, arg):
         if arg:
             self.ctl.output('Error: reload accepts no arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_reload()
             return
 
@@ -949,7 +1014,8 @@ class DefaultControllerPlugin(ControllerPluginBase):
             supervisor = self.ctl.get_supervisor()
             try:
                 supervisor.restart()
-            except xmlrpclib.Fault, e:
+            except xmlrpclib.Fault as e:
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                     self.ctl.output('ERROR: already shutting down')
                 else:
@@ -996,13 +1062,15 @@ class DefaultControllerPlugin(ControllerPluginBase):
     def do_avail(self, arg):
         if arg:
             self.ctl.output('Error: avail accepts no arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_avail()
             return
 
         supervisor = self.ctl.get_supervisor()
         try:
             configinfo = supervisor.getAllConfigInfo()
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                 self.ctl.output('ERROR: supervisor shutting down')
             else:
@@ -1017,17 +1085,19 @@ class DefaultControllerPlugin(ControllerPluginBase):
     def do_reread(self, arg):
         if arg:
             self.ctl.output('Error: reread accepts no arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_reread()
             return
 
         supervisor = self.ctl.get_supervisor()
         try:
             result = supervisor.reloadConfig()
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                 self.ctl.output('ERROR: supervisor shutting down')
             elif e.faultCode == xmlrpc.Faults.CANT_REREAD:
-                self.ctl.output('ERROR: %s' % e.faultString)
+                self.ctl.output("ERROR: %s" % e.faultString)
             else:
                 raise
         else:
@@ -1043,15 +1113,17 @@ class DefaultControllerPlugin(ControllerPluginBase):
         for name in names:
             try:
                 supervisor.addProcessGroup(name)
-            except xmlrpclib.Fault, e:
+            except xmlrpclib.Fault as e:
                 if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                     self.ctl.output('ERROR: shutting down')
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 elif e.faultCode == xmlrpc.Faults.ALREADY_ADDED:
                     self.ctl.output('ERROR: process group already active')
                 elif e.faultCode == xmlrpc.Faults.BAD_NAME:
-                    self.ctl.output(
-                        "ERROR: no such process/group: %s" % name)
+                    self.ctl.output("ERROR: no such process/group: %s" % name)
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 else:
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                     raise
             else:
                 self.ctl.output("%s: added process group" % name)
@@ -1067,13 +1139,13 @@ class DefaultControllerPlugin(ControllerPluginBase):
         for name in names:
             try:
                 supervisor.removeProcessGroup(name)
-            except xmlrpclib.Fault, e:
+            except xmlrpclib.Fault as e:
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 if e.faultCode == xmlrpc.Faults.STILL_RUNNING:
                     self.ctl.output('ERROR: process/group still running: %s'
-                                    % name)
+                                      % name)
                 elif e.faultCode == xmlrpc.Faults.BAD_NAME:
-                    self.ctl.output(
-                        "ERROR: no such process/group: %s" % name)
+                    self.ctl.output("ERROR: no such process/group: %s" % name)
                 else:
                     raise
             else:
@@ -1090,7 +1162,8 @@ class DefaultControllerPlugin(ControllerPluginBase):
         supervisor = self.ctl.get_supervisor()
         try:
             result = supervisor.reloadConfig()
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             if e.faultCode == xmlrpc.Faults.SHUTDOWN_STATE:
                 self.ctl.output('ERROR: already shutting down')
                 return
@@ -1117,6 +1190,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
             for gname in valid_gnames:
                 if gname not in groups:
                     self.ctl.output('ERROR: no such group: %s' % gname)
+                    self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
 
         for gname in removed:
             if valid_gnames and gname not in valid_gnames:
@@ -1127,7 +1201,8 @@ class DefaultControllerPlugin(ControllerPluginBase):
             fails = [res for res in results
                      if res['status'] == xmlrpc.Faults.FAILED]
             if fails:
-                log(gname, "has problems; not removing")
+                self.ctl.output("%s: %s" % (gname, "has problems; not removing"))
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
                 continue
             supervisor.removeProcessGroup(gname)
             log(gname, "removed process group")
@@ -1135,7 +1210,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
         for gname in changed:
             if valid_gnames and gname not in valid_gnames:
                 continue
-            results = supervisor.stopProcessGroup(gname)
+            supervisor.stopProcessGroup(gname)
             log(gname, "stopped")
 
             supervisor.removeProcessGroup(gname)
@@ -1173,6 +1248,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         if not names:
             self.ctl.output('Error: clear requires a process name')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_clear()
             return
 
@@ -1181,19 +1257,20 @@ class DefaultControllerPlugin(ControllerPluginBase):
         if 'all' in names:
             results = supervisor.clearAllProcessLogs()
             for result in results:
-                result = self._clearresult(result)
-                self.ctl.output(result)
+                self.ctl.output(self._clearresult(result))
+                self.ctl.set_exitstatus_from_xmlrpc_fault(result['status'])
         else:
             for name in names:
                 group_name, process_name = split_namespec(name)
                 try:
-                    result = supervisor.clearProcessLogs(name)
-                except xmlrpclib.Fault, e:
-                    error = self._clearresult({'status': e.faultCode,
-                                               'name': process_name,
-                                               'group': group_name,
-                                               'description': e.faultString})
-                    self.ctl.output(error)
+                    supervisor.clearProcessLogs(name)
+                except xmlrpclib.Fault as e:
+                    error = {'status': e.faultCode,
+                             'name': process_name,
+                             'group': group_name,
+                             'description': e.faultString}
+                    self.ctl.output(self._clearresult(error))
+                    self.ctl.set_exitstatus_from_xmlrpc_fault(error['status'])
                 else:
                     name = make_namespec(group_name, process_name)
                     self.ctl.output('%s: cleared' % name)
@@ -1209,9 +1286,13 @@ class DefaultControllerPlugin(ControllerPluginBase):
         parts = urlparse.urlparse(url)
         if parts[0] not in ('unix', 'http'):
             self.ctl.output('ERROR: url must be http:// or unix://')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             return
         self.ctl.options.serverurl = url
+        # TODO review this
+        old_exitstatus = self.ctl.exitstatus
         self.do_status('')
+        self.ctl.exitstatus = old_exitstatus
 
     def help_open(self):
         self.ctl.output("open <url>\tConnect to a remote supervisord process.")
@@ -1220,6 +1301,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
     def do_version(self, arg):
         if arg:
             self.ctl.output('Error: version accepts no arguments')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_version()
             return
 
@@ -1240,10 +1322,12 @@ class DefaultControllerPlugin(ControllerPluginBase):
         names = arg.split()
         if not names:
             self.ctl.output('ERROR: no process name supplied')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             self.help_fg()
             return
         if len(names) > 1:
             self.ctl.output('ERROR: too many process names supplied')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             return
 
         name = names[0]
@@ -1251,15 +1335,17 @@ class DefaultControllerPlugin(ControllerPluginBase):
 
         try:
             info = supervisor.getProcessInfo(name)
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault as e:
             if e.faultCode == xmlrpc.Faults.BAD_NAME:
                 self.ctl.output('ERROR: bad process name supplied')
+                self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             else:
                 self.ctl.output('ERROR: ' + str(e))
             return
 
         if info['state'] != states.ProcessStates.RUNNING:
             self.ctl.output('ERROR: process not running')
+            self.ctl.exitstatus = LSBInitExitStatuses.GENERIC
             return
 
         self.ctl.output('==> Press Ctrl-C to exit <==')
@@ -1275,7 +1361,7 @@ class DefaultControllerPlugin(ControllerPluginBase):
                 inp = raw_input() + '\n'
                 try:
                     supervisor.sendProcessStdin(name, inp)
-                except xmlrpclib.Fault, e:
+                except xmlrpclib.Fault as e:
                     if e.faultCode == xmlrpc.Faults.NOT_RUNNING:
                         self.ctl.output('Process got killed')
                     else:
@@ -1309,10 +1395,11 @@ def main(args=None, options=None):
 
     if options.args:
         c.onecmd(" ".join(options.args))
+        sys.exit(c.exitstatus)
 
     if options.interactive:
         c.exec_cmdloop(args, options)
-
+        sys.exit(0)  # exitstatus always 0 for interactive mode
 
 if __name__ == "__main__":
     main()
