@@ -1,13 +1,21 @@
-import types
-import socket
-import xmlrpclib
-import httplib
-import urllib
+import datetime
 import re
-from cStringIO import StringIO
-import traceback
+import socket
 import sys
-import base64
+import time
+import traceback
+import types
+from xml.etree.ElementTree import iterparse
+
+from supervisor.compat import xmlrpclib
+from supervisor.compat import StringIO
+from supervisor.compat import urllib
+from supervisor.compat import as_bytes
+from supervisor.compat import as_string
+from supervisor.compat import encodestring
+from supervisor.compat import decodestring
+from supervisor.compat import httplib
+from supervisor.compat import PY2
 
 from supervisor.medusa.http_server import get_header
 from supervisor.medusa.xmlrpc_handler import xmlrpc_handler
@@ -70,7 +78,7 @@ class DeferredXMLRPCResponse:
                 value = self.callback()
                 if value is NOT_DONE_YET:
                     return NOT_DONE_YET
-            except RPCError, err:
+            except RPCError as err:
                 value = xmlrpclib.Fault(err.code, err.text)
 
             body = xmlrpc_marshal(value)
@@ -94,28 +102,15 @@ class DeferredXMLRPCResponse:
         connection = get_header(self.CONNECTION, self.request.header)
 
         close_it = 0
-        wrap_in_chunking = 0
 
         if self.request.version == '1.0':
             if connection == 'keep-alive':
-                if not self.request.has_key ('Content-Length'):
-                    close_it = 1
-                else:
-                    self.request['Connection'] = 'Keep-Alive'
+                self.request['Connection'] = 'Keep-Alive'
             else:
                 close_it = 1
         elif self.request.version == '1.1':
             if connection == 'close':
                 close_it = 1
-            elif not self.request.has_key ('Content-Length'):
-                if self.request.has_key ('Transfer-Encoding'):
-                    if not self.request['Transfer-Encoding'] == 'chunked':
-                        close_it = 1
-                elif self.request.use_chunked:
-                    self.request['Transfer-Encoding'] = 'chunked'
-                    wrap_in_chunking = 1
-                else:
-                    close_it = 1
         elif self.request.version is None:
             close_it = 1
 
@@ -125,19 +120,9 @@ class DeferredXMLRPCResponse:
         if close_it:
             self.request['Connection'] = 'close'
 
-        if wrap_in_chunking:
-            outgoing_producer = producers.chunked_producer (
-                    producers.composite_producer (self.request.outgoing)
-                    )
-            # prepend the header
-            outgoing_producer = producers.composite_producer(
-                [outgoing_header, outgoing_producer]
-                )
-        else:
-            # prepend the header
-            self.request.outgoing.insert(0, outgoing_header)
-            outgoing_producer = producers.composite_producer (
-                self.request.outgoing)
+        # prepend the header
+        self.request.outgoing.insert(0, outgoing_header)
+        outgoing_producer = producers.composite_producer(self.request.outgoing)
 
         # apply a few final transformations to the output
         self.request.channel.push_with_producer (
@@ -161,7 +146,7 @@ def xmlrpc_marshal(value):
     if ismethodresponse:
         if not isinstance(value, tuple):
             value = (value,)
-        body = xmlrpclib.dumps(value,  methodresponse=ismethodresponse)
+        body = xmlrpclib.dumps(value, methodresponse=ismethodresponse)
     else:
         body = xmlrpclib.dumps(value)
     return body
@@ -181,8 +166,7 @@ class SystemNamespaceRPCInterface:
                 # introspect; any methods that don't start with underscore
                 # are published
                 func = getattr(namespace, method_name)
-                meth = getattr(func, 'im_func', None)
-                if meth is not None:
+                if callable(func):
                     if not method_name.startswith('_'):
                         sig = '%s.%s' % (ns_name, method_name)
                         methods[sig] = str(func.__doc__)
@@ -194,7 +178,7 @@ class SystemNamespaceRPCInterface:
         @return array result  An array of method names available (strings).
         """
         methods = self._listMethods()
-        keys = methods.keys()
+        keys = list(methods.keys())
         keys.sort()
         return keys
 
@@ -260,7 +244,7 @@ class SystemNamespaceRPCInterface:
             if callbacks:
                 try:
                     value = callbacks[0]()
-                except RPCError, exc:
+                except RPCError as exc:
                     value = {'faultCode': exc.code,
                              'faultString': exc.text}
                 except:
@@ -289,7 +273,7 @@ class SystemNamespaceRPCInterface:
                     # make the call, may return a callback or not
                     root = AttrDict(self.namespaces)
                     value = traverse(root, name, params)
-                except RPCError, exc:
+                except RPCError as exc:
                     value = {'faultCode': exc.code,
                              'faultString': exc.text}
                 except:
@@ -344,31 +328,75 @@ def make_datetime(text):
 class supervisor_xmlrpc_handler(xmlrpc_handler):
     path = '/RPC2'
     IDENT = 'Supervisor XML-RPC Handler'
+
+    unmarshallers = {
+        "int": lambda x: int(x.text),
+        "i4": lambda x: int(x.text),
+        "boolean": lambda x: x.text == "1",
+        "string": lambda x: x.text or "",
+        "double": lambda x: float(x.text),
+        "dateTime.iso8601": lambda x: make_datetime(x.text),
+        "array": lambda x: x[0].text,
+        "data": lambda x: [v.text for v in x],
+        "struct": lambda x: dict([(k.text or "", v.text) for k, v in x]),
+        "base64": lambda x: as_string(decodestring(as_bytes(x.text or ""))),
+        "param": lambda x: x[0].text,
+        }
+
     def __init__(self, supervisord, subinterfaces):
         self.rpcinterface = RootRPCInterface(subinterfaces)
         self.supervisord = supervisord
-        if loads:
-            self.loads = loads
-        else:
-            self.supervisord.options.logger.warn(
-                'cElementTree not installed, using slower XML parser for '
-                'XML-RPC'
-                )
-            self.loads = xmlrpclib.loads
+
+    def loads(self, data):
+        params = method = None
+        for action, elem in iterparse(StringIO(data)):
+            unmarshall = self.unmarshallers.get(elem.tag)
+            if unmarshall:
+                data = unmarshall(elem)
+                elem.clear()
+                elem.text = data
+            elif elem.tag == "value":
+                try:
+                    data = elem[0].text
+                except IndexError:
+                    data = elem.text or ""
+                elem.clear()
+                elem.text = data
+            elif elem.tag == "methodName":
+                method = elem.text
+            elif elem.tag == "params":
+                params = tuple([v.text for v in elem])
+        return params, method
 
     def match(self, request):
         return request.uri.startswith(self.path)
 
-    def continue_request (self, data, request):
+    def continue_request(self, data, request):
         logger = self.supervisord.options.logger
 
         try:
-
-            params, method = self.loads(data)
+            try:
+                # on 2.x, the Expat parser doesn't like Unicode which actually
+                # contains non-ASCII characters. It's a bit of a kludge to
+                # do it conditionally here, but it's down to how underlying
+                # libs behave
+                if PY2:
+                    data = data.encode('ascii', 'xmlcharrefreplace')
+                params, method = self.loads(data)
+            except:
+                logger.error(
+                    'XML-RPC request data %r is invalid: unmarshallable' %
+                    (data,)
+                )
+                request.error(400)
+                return
 
             # no <methodName> in the request or name is an empty string
             if not method:
-                logger.trace('XML-RPC request received with no method name')
+                logger.error(
+                    'XML-RPC request data %r is invalid: no method name' %
+                    (data,)
+                    )
                 request.error(400)
                 return
 
@@ -380,17 +408,9 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
             try:
                 logger.trace('XML-RPC method called: %s()' % method)
                 value = self.call(method, params)
-                # application-specific: instead of we never want to
-                # marshal None (even though we could by saying allow_none=True
-                # in dumps within xmlrpc_marshall), this is meant as
-                # a debugging fixture, see issue 223.
-                assert value is not None, (
-                    'return value from method %r with params %r is None' %
-                    (method, params)
-                    )
                 logger.trace('XML-RPC method %s() returned successfully' %
                              method)
-            except RPCError, err:
+            except RPCError as err:
                 # turn RPCError reported by method into a Fault instance
                 value = xmlrpclib.Fault(err.code, err.text)
                 logger.trace('XML-RPC method %s() returned fault: [%d] %s' % (
@@ -407,7 +427,7 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
                 # if we get anything but a function, it implies that this
                 # response doesn't need to be deferred, we can service it
                 # right away.
-                body = xmlrpc_marshal(value)
+                body = as_bytes(xmlrpc_marshal(value))
                 request['Content-Type'] = 'text/xml'
                 request['Content-Length'] = len(body)
                 request.push(body)
@@ -415,7 +435,10 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
 
         except:
             tb = traceback.format_exc()
-            logger.critical(tb)
+            logger.critical(
+                "Handling XML-RPC request with data %r raised an unexpected "
+                "exception: %s" % (data, tb)
+                )
             # internal error, report as HTTP server error
             request.error(500)
 
@@ -423,18 +446,27 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
         return traverse(self.rpcinterface, method, params)
 
 def traverse(ob, method, params):
-    path = method.split('.')
-    for name in path:
-        if name.startswith('_'):
-            # security (don't allow things that start with an underscore to
-            # be called remotely)
-            raise RPCError(Faults.UNKNOWN_METHOD)
-        ob = getattr(ob, name, None)
-        if ob is None:
-            raise RPCError(Faults.UNKNOWN_METHOD)
+    dotted_parts = method.split('.')
+    # security (CVE-2017-11610, don't allow object traversal)
+    if len(dotted_parts) != 2:
+        raise RPCError(Faults.UNKNOWN_METHOD)
+    namespace, method = dotted_parts
+
+    # security (don't allow methods that start with an underscore to
+    # be called remotely)
+    if method.startswith('_'):
+        raise RPCError(Faults.UNKNOWN_METHOD)
+
+    rpcinterface = getattr(ob, namespace, None)
+    if rpcinterface is None:
+        raise RPCError(Faults.UNKNOWN_METHOD)
+
+    func = getattr(rpcinterface, method, None)
+    if not isinstance(func, types.MethodType):
+        raise RPCError(Faults.UNKNOWN_METHOD)
 
     try:
-        return ob(*params)
+        return func(*params)
     except TypeError:
         raise RPCError(Faults.INCORRECT_PARAMETERS)
 
@@ -447,8 +479,8 @@ class SupervisorTransport(xmlrpclib.Transport):
     """
     connection = None
 
-    _use_datetime = 0 # python 2.5 fwd compatibility
     def __init__(self, username=None, password=None, serverurl=None):
+        xmlrpclib.Transport.__init__(self)
         self.username = username
         self.password = password
         self.verbose = False
@@ -475,7 +507,13 @@ class SupervisorTransport(xmlrpclib.Transport):
         else:
             raise ValueError('Unknown protocol for serverurl %s' % serverurl)
 
+    def close(self):
+        if self.connection:
+            self.connection.close()
+            self.connection = None
+
     def request(self, host, handler, request_body, verbose=0):
+        request_body = as_bytes(request_body)
         if not self.connection:
             self.connection = self._get_connection()
             self.headers = {
@@ -487,7 +525,9 @@ class SupervisorTransport(xmlrpclib.Transport):
             # basic auth
             if self.username is not None and self.password is not None:
                 unencoded = "%s:%s" % (self.username, self.password)
-                encoded = base64.encodestring(unencoded).replace('\n', '')
+                encoded = as_string(encodestring(as_bytes(unencoded)))
+                encoded = encoded.replace('\n', '')
+                encoded = encoded.replace('\012', '')
                 self.headers["Authorization"] = "Basic %s" % encoded
 
         self.headers["Content-Length"] = str(len(request_body))
@@ -504,13 +544,17 @@ class SupervisorTransport(xmlrpclib.Transport):
                                           r.reason,
                                           '' )
         data = r.read()
+        data = as_string(data)
+        # on 2.x, the Expat parser doesn't like Unicode which actually
+        # contains non-ASCII characters
+        data = data.encode('ascii', 'xmlcharrefreplace')
         p, u = self.getparser()
         p.feed(data)
         p.close()
         return u.close()
 
 class UnixStreamHTTPConnection(httplib.HTTPConnection):
-    def connect(self):
+    def connect(self): # pragma: no cover
         self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         # we abuse the host parameter as the socketname
         self.sock.connect(self.socketfile)
@@ -552,61 +596,10 @@ def gettags(comment):
         else:
             if line:
                 tag_text.append(line)
-        lineno = lineno + 1
+        lineno += 1
 
     tags.append((tag_lineno, tag, datatype, name, '\n'.join(tag_text)))
 
     return tags
 
 
-try:
-    # Python 2.6 contains a version of cElementTree inside it.
-    from xml.etree.ElementTree import iterparse
-except ImportError:
-    try:
-        # Failing that, try cElementTree instead.
-        from cElementTree import iterparse
-    except ImportError:
-        iterparse = None
-
-
-if iterparse is not None:
-    import datetime, time
-    from base64 import decodestring
-
-    unmarshallers = {
-        "int": lambda x: int(x.text),
-        "i4": lambda x: int(x.text),
-        "boolean": lambda x: x.text == "1",
-        "string": lambda x: x.text or "",
-        "double": lambda x: float(x.text),
-        "dateTime.iso8601": lambda x: make_datetime(x.text),
-        "array": lambda x: x[0].text,
-        "data": lambda x: [v.text for v in x],
-        "struct": lambda x: dict([(k.text or "", v.text) for k, v in x]),
-        "base64": lambda x: decodestring(x.text or ""),
-        "param": lambda x: x[0].text,
-    }
-
-    def loads(data):
-        params = method = None
-        for action, elem in iterparse(StringIO(data)):
-            unmarshal = unmarshallers.get(elem.tag)
-            if unmarshal:
-                data = unmarshal(elem)
-                elem.clear()
-                elem.text = data
-            elif elem.tag == "value":
-                try:
-                    data = elem[0].text
-                except IndexError:
-                    data = elem.text or ""
-                elem.clear()
-                elem.text = data
-            elif elem.tag == "methodName":
-                method = elem.text
-            elif elem.tag == "params":
-                params = tuple([v.text for v in elem])
-        return params, method
-else:
-    loads = None
